@@ -107,13 +107,22 @@ const WindowContextMenu = new Lang.Class({
                                      Lang.bind(this, this._updateMaximizeItem));
         this._updateMaximizeItem();
 
-        let item = new PopupMenu.PopupMenuItem(_("Close"));
-        item.connect('activate', Lang.bind(this, function() {
+        this._closeItem = new PopupMenu.PopupMenuItem(_("Close"));
+        this._closeItem.connect('activate', Lang.bind(this, function() {
             this._metaWindow.delete(global.get_current_time());
         }));
-        this.addMenuItem(item);
+        this.addMenuItem(this._closeItem);
 
         this.actor.connect('destroy', Lang.bind(this, this._onDestroy));
+
+        this.connect('open-state-changed', () => {
+            if (!this.isOpen)
+                return;
+
+            this._minimizeItem.setSensitive(this._metaWindow.can_minimize());
+            this._maximizeItem.setSensitive(this._metaWindow.can_maximize());
+            this._closeItem.setSensitive(this._metaWindow.can_close());
+        });
     },
 
     _updateMinimizeItem: function() {
@@ -781,7 +790,7 @@ const WindowList = new Lang.Class({
         this._monitor = monitor;
 
         this.actor = new St.Widget({ name: 'panel',
-                                     style_class: 'bottom-panel',
+                                     style_class: 'bottom-panel solid',
                                      reactive: true,
                                      track_hover: true,
                                      layout_manager: new Clutter.BinLayout()});
@@ -813,10 +822,15 @@ const WindowList = new Lang.Class({
         this._workspaceIndicator = new WorkspaceIndicator();
         indicatorsBox.add(this._workspaceIndicator.container, { expand: false, y_fill: true });
 
+        this._mutterSettings = new Gio.Settings({ schema_id: 'org.gnome.mutter' });
         this._workspaceSettings = this._getWorkspaceSettings();
         this._workspacesOnlyOnPrimaryChangedId =
             this._workspaceSettings.connect('changed::workspaces-only-on-primary',
                                             Lang.bind(this, this._updateWorkspaceIndicatorVisibility));
+        this._dynamicWorkspacesSettings = this._getDynamicWorkspacesSettings();
+        this._dynamicWorkspacesChangedId =
+            this._dynamicWorkspacesSettings.connect('changed::dynamic-workspaces',
+                                                    Lang.bind(this, this._updateWorkspaceIndicatorVisibility));
         this._updateWorkspaceIndicatorVisibility();
 
         this._menuManager = new PopupMenu.PopupMenuManager(this);
@@ -898,11 +912,17 @@ const WindowList = new Lang.Class({
         this._groupingModeChanged();
     },
 
+    _getDynamicWorkspacesSettings: function() {
+        if (this._workspaceSettings.list_keys().indexOf('dynamic-workspaces') > -1)
+            return this._workspaceSettings;
+        return this._mutterSettings;
+    },
+
     _getWorkspaceSettings: function() {
         let settings = global.get_overrides_settings();
         if (settings.list_keys().indexOf('workspaces-only-on-primary') > -1)
             return settings;
-        return new Gio.Settings({ schema_id: 'org.gnome.mutter' });
+        return this._mutterSettings;
     },
 
     _onScrollEvent: function(actor, event) {
@@ -936,9 +956,12 @@ const WindowList = new Lang.Class({
     },
 
     _updateWorkspaceIndicatorVisibility: function() {
-        this._workspaceIndicator.actor.visible =
-            this._monitor == Main.layoutManager.primaryMonitor ||
-            !this._workspaceSettings.get_boolean('workspaces-only-on-primary');
+        let hasWorkspaces = this._dynamicWorkspacesSettings.get_boolean('dynamic-workspaces') ||
+                            global.screen.n_workspaces > 1;
+        let workspacesOnMonitor = this._monitor == Main.layoutManager.primaryMonitor ||
+                                  !this._workspaceSettings.get_boolean('workspaces-only-on-primary');
+
+        this._workspaceIndicator.actor.visible = hasWorkspaces && workspacesOnMonitor;
     },
 
     _getPreferredUngroupedWindowListWidth: function() {
@@ -1109,6 +1132,8 @@ const WindowList = new Lang.Class({
                                   Lang.bind(this, this._onWindowRemoved));
             this._workspaceSignals.set(workspace, signals);
         }
+
+        this._updateWorkspaceIndicatorVisibility();
     },
 
     _disconnectWorkspaceSignals: function() {
@@ -1177,6 +1202,7 @@ const WindowList = new Lang.Class({
 
     _onDestroy: function() {
         this._workspaceSettings.disconnect(this._workspacesOnlyOnPrimaryChangedId);
+        this._dynamicWorkspacesSettings.disconnect(this._dynamicWorkspacesChangedId);
 
         this._workspaceIndicator.destroy();
 
